#!/usr/bin/python
#
# Copyright 2014 Canonical Ltd.  All rights reserved
# Author: Chris Stratford <chris.stratford@canonical.com>

import sys
import os
import os.path
import shutil
sys.path.insert(0, os.path.join(os.environ['CHARM_DIR'], 'lib'))
from charmhelpers.core.hookenv import Hooks, log, charm_dir, relation_set, relation_id, relation_ids
from charmhelpers.core.host import mkdir
from charmhelpers.fetch import apt_install, apt_update
from charmhelpers.contrib.charmsupport.nrpe import NRPE
from Cheetah.Template import Template
from Config import Config

hooks = Hooks()
conf = Config()

requiredPkgs = [
    "rkhunter",
]

scriptsToCopy = [
    "check_rkhunter_output.py",
]

def jujuHeader():
    header = ("#-------------------------------------------------#\n"
              "# This file is Juju managed - do not edit by hand #\n"
              "#-------------------------------------------------#\n")
    return header

def fileFromTemplate(tmpl, dest, searchList):
    template_file = os.path.join(charm_dir(), "templates", tmpl)
    t = Template(file=template_file, searchList=searchList)
    with open(dest, "w") as f:
        f.write(jujuHeader())
        f.write(str(t))
    os.chmod(dest, 0444)


def copyFiles():
    mkdir(conf.scriptDir())
    for script in scriptsToCopy:
        src = os.path.join(charm_dir(), "files", script)
        dest = os.path.join(conf.scriptDir(), script)
        shutil.copyfile(src, dest)
        os.chmod(dest, 0755)


def configureNrpe():
    nrpeCompat = NRPE()
    nrpeCompat.add_check(
        shortname="rkhunter",
        description="Check rkhunter output",
        check_cmd=os.path.join(conf.scriptDir(), "check_rkhunter_output.py")
    )
    nrpeCompat.write()


def updateLogrotate():
    relation_data = {}
    relation_data["logfiles"] = '{"rkhunter":{"path":conf.logFile(),"when":"daily"}}'
    if relation_id():
        relation_set(None, relation_data)
    else:
        for r in relation_ids("logrotate"):
            relation_set(r, relation_data)


def installPackages():
    apt_update()
    apt_install(requiredPkgs, options=['--force-yes'])


def writeConfig():
    tmpl_data = {}
    tmpl_data["logFile"] = conf.logFile()
    tmpl_data["devFiles"] = conf.devFiles()
    tmpl_data["hiddenDirs"] = conf.hiddenDirs()
    tmpl_data["hiddenFiles"] = conf.hiddenFiles()
    tmpl_data["scriptWhitelist"] = conf.scriptWhitelist()
    tmpl_data["sshRootAllowed"] = conf.sshRootAllowed()
    tmpl_data["mirrorAccess"] = conf.mirrorAccess()
    tmpl_data["mirrorMode"] = conf.mirrorMode()
    tmpl_data["mirrorUpdate"] = conf.mirrorUpdate()
    config_path = "/etc/rkhunter.conf.local"
    fileFromTemplate("rkhunter-conf.tmpl", config_path, tmpl_data)
    os.chmod(config_path, 0444)


def updateDefaults():
    origFile = "/etc/default/rkhunter"
    tmpFile = origFile + ".tmp"
    with open(origFile, "r") as fr:
        with open(tmpFile, "w") as fw:
            for line in fr:
                if "REPORT_EMAIL=" in line:
                    fw.write('REPORT_EMAIL="{}"\n'.format(conf.mailRecipient()))
                elif "CRON_DAILY_RUN=" in line:
                    fw.write('CRON_DAILY_RUN="{}"\n'.format(conf.dailyRun()))
                elif "CRON_DB_UPDATE=" in line:
                    fw.write('CRON_DB_UPDATE="{}"\n'.format(conf.weeklyDbUpdate()))
                else:
                    fw.write(line)
    os.rename(tmpFile, origFile)


def writeMirrorList():
    listFile = "/var/lib/rkhunter/db/mirrors.dat"
    tmpFile = listFile + ".tmp"
    with open(tmpFile, "w") as f:
        f.write("Version:1970010101\n")
        for item in conf.mirrorList():
            f.write(item + "\n")
    os.rename(tmpFile, listFile)


def configure():
    log("CHARM: configuring {}".format(conf.app_name()))
    mkdir(os.path.dirname(conf.logFile()))
    updateDefaults()
    writeMirrorList()
    writeConfig()


@hooks.hook("logrotate-relation-changed")
def logrotateRelationChanged():
    log("CHARM: Changing logrotate relation for {}".format(conf.app_name()))
    updateLogrotate()


@hooks.hook("nrpe-external-master-relation-changed")
def nrpeExternalMasterRelationChanged():
    log("CHARM: Changing nrpe-external-master relation for {}".format(conf.app_name()))
    configureNrpe()


@hooks.hook("install")
def install():
    log("CHARM: Installing {}".format(conf.app_name()))
    installPackages()
    copyFiles()


@hooks.hook("upgrade-charm")
def upgrade_charm():
    log("CHARM: Upgrading {}".format(conf.app_name()))
    copyFiles()


@hooks.hook("config-changed")
def config_changed():
    log("CHARM: Configuring {}".format(conf.app_name()))
    configure()
    configureNrpe()


@hooks.hook("start")
def start():
    # Nothing to start - just here to keep charm proof happy
    log("CHARM: Starting {}".format(conf.app_name()))


@hooks.hook("stop")
def stop():
    # Nothing to stop - just here to keep charm proof happy
    log("CHARM: Stopping {}".format(conf.app_name()))


if __name__ == "__main__":
    hooks.execute(sys.argv)
